=== modified file 'bzrlib/benchmarks/bench_workingtree.py'
--- bzrlib/benchmarks/bench_workingtree.py	2006-10-11 23:08:27 +0000
+++ bzrlib/benchmarks/bench_workingtree.py	2006-11-13 14:13:56 +0000
@@ -20,6 +20,7 @@
 
 from bzrlib.benchmarks import Benchmark
 from bzrlib.workingtree import WorkingTree
+from bzrlib import ignores
 
 
 class WorkingTreeBenchmark(Benchmark):
@@ -54,3 +55,26 @@
             for x in xrange(10824):
                 t.is_ignored(str(x))
         self.time(call_is_ignored_10824_not_ignored)
+
+    def test_is_ignored_10_patterns(self):
+        t = self.make_branch_and_tree('.')
+        ignores.add_runtime_ignores([u'*.%i' % i for i in range(1, 9)])
+        ignores.add_runtime_ignores(['./foo', 'foo/bar'])
+        self.time(t.is_ignored,'bar')
+        ignores._runtime_ignores = set()
+
+    def test_is_ignored_100_patterns(self):
+        t = self.make_branch_and_tree('.')
+        ignores.add_runtime_ignores([u'*.%i' % i for i in range(1, 99)])
+        ignores.add_runtime_ignores(['./foo', 'foo/bar'])
+        self.time(t.is_ignored,'bar')
+        ignores._runtime_ignores = set()
+
+    def test_is_ignored_1000_patterns(self):
+        t = self.make_branch_and_tree('.')
+        ignores.add_runtime_ignores([u'*.%i' % i for i in range(1, 999)])
+        ignores.add_runtime_ignores(['./foo', 'foo/bar'])
+        self.time(t.is_ignored,'bar')
+        ignores._runtime_ignores = set()
+
+

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-11-11 13:04:54 +0000
+++ bzrlib/builtins.py	2006-11-13 14:13:56 +0000
@@ -1586,7 +1586,9 @@
 
     Ignore patterns are case-insensitive on case-insensitive systems.
 
-    Note: wildcards must be quoted from the shell on Unix.
+    Ignore patterns may include shell-style globbing wildcards such as
+    '*' and '?', and character grouping such as [a-z].
+    Note: shell globs must be quoted from the shell on Unix.
 
     examples:
         bzr ignore ./Makefile

=== added file 'bzrlib/glob.py'
--- bzrlib/glob.py	1970-01-01 00:00:00 +0000
+++ bzrlib/glob.py	2006-11-13 14:13:56 +0000
@@ -0,0 +1,224 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tools for converting globs to regular expressions.
+
+This module provides functions for converting shell-like globs to regular
+expressions.
+"""
+
+import re
+
+from bzrlib.trace import (
+    mutter, 
+    warning
+    )
+
+
+class Replacer(object):
+    """Do a multiple-pattern substitution.
+
+    The patterns and substitutions are combined into one, so the result of
+    one replacement is never substituted again. Add the patterns and
+    replacements via the add method and then call the object. The patterns
+    must not contain capturing groups.
+    """
+
+    _expand = re.compile(ur'\\&')
+
+    def __init__(self, source=None):
+        self._pat = None
+        if source:
+            self._pats = list(source._pats)
+            self._funs = list(source._funs)
+        else:
+            self._pats = []
+            self._funs = []
+
+    def add(self, pat, fun):
+        r"""Add a pattern and replacement.
+
+        The pattern must not contain capturing groups.
+        The replacement might be either a string template in which \& will be
+        replaced with the match, or a function that will get the matching text  
+        as argument. It does not get match object, because capturing is 
+        forbidden anyway.
+        """
+        self._pat = None
+        self._pats.append(pat)
+        self._funs.append(fun)
+
+    def add_replacer(self, replacer):
+        r"""Add all patterns from another replacer.
+
+        All patterns and replacements from replacer are appended to the ones
+        already defined.
+        """
+        self._pat = None
+        self._pats.extend(replacer._pats)
+        self._funs.extend(replacer._funs)
+
+    def __call__(self, text):
+        if not self._pat:
+            self._pat = re.compile(
+                    u'|'.join([u'(%s)' % p for p in self._pats]),
+                    re.UNICODE)
+        return self._pat.sub(self._do_sub, text)
+
+    def _do_sub(self, m):
+        fun = self._funs[m.lastindex - 1]
+        if hasattr(fun, '__call__'):
+            return fun(m.group(0))
+        else:
+            return self._expand.sub(m.group(0), fun)
+
+
+_sub_named = Replacer()
+_sub_named.add(ur'\[:digit:\]', ur'\d')
+_sub_named.add(ur'\[:space:\]', ur'\s')
+_sub_named.add(ur'\[:alnum:\]', ur'\w')
+_sub_named.add(ur'\[:ascii:\]', ur'\0-\x7f')
+_sub_named.add(ur'\[:blank:\]', ur' \t')
+_sub_named.add(ur'\[:cntrl:\]', ur'\0-\x1f\x7f-\x9f')
+
+
+_sub_leading_named = Replacer()
+_sub_leading_named.add(ur'\[:ascii:\]', ur'\0-\x2d\x2f-\x7f')
+_sub_leading_named.add_replacer(_sub_named)
+
+
+def _sub_group(m):
+    if m[1] in (u'!', u'^'):
+        return u'[^' + _sub_named(m[2:-1]) + u']'
+    return u'[' + _sub_named(m[1:-1]) + u']'
+
+
+def _sub_leading_group(m):
+    if m[1] in (u'!', u'^'):
+        return u'[^.' + _sub_named(m[2:-1]) + u']'
+    return u'[' + _sub_leading_named(m[1:-1]) + u']'
+        
+
+def _invalid_regex(repl):
+    def _(m):
+        warning(u"'%s' not allowed withing regexp. Replacing with '%s'" %
+                (m, repl))
+        return repl
+    return _
+
+
+_sub_re = Replacer()
+_sub_re.add(u'^RE:', u'')
+_sub_re.add(u'\((?!\?)', u'(?:')
+_sub_re.add(u'\(\?P<.*>', _invalid_regex(u'(?:'))
+_sub_re.add(u'\(\?P=[^)]*\)', _invalid_regex(u''))
+
+
+_sub_shell = Replacer()
+_sub_shell.add(ur'^RE:.*', _sub_re) # RE:<anything> is a regex
+_sub_shell.add(ur'(?:(?<=/)|^)\[\^?\]?(?:[^][]|\[:[^]]+:\])+\]', 
+    _sub_leading_group) # char group
+_sub_shell.add(ur'\[\^?\]?(?:[^][]|\[:[^]]+:\])+\]', _sub_group) # char group
+_sub_shell.add(ur'(?:(?<=/)|^)(?:\.?/)+', u'') # canonicalize path
+_sub_shell.add(ur'\\.', ur'\&') # keep anything backslashed
+_sub_shell.add(ur'[(){}|^$+.]', ur'\\&') # escape specials
+_sub_shell.add(ur'(?:(?<=/)|^)\*\.', ur'[^./][^/]*\.') # *. after /|^
+_sub_shell.add(ur'(?:(?<=/)|^)\*', ur'(?:[^./][^/]*)?') # * after /|^
+_sub_shell.add(ur'\*', ur'[^/]*') # * elsewhere
+_sub_shell.add(ur'(?:(?<=/)|^)\?', ur'[^./]') # ? after /|^
+_sub_shell.add(ur'\?', ur'[^/]') # ? elsewhere
+
+
+_sub_shell_basename = Replacer()
+_sub_shell_basename.add(ur'^\[\^?\]?(?:[^][]|\[:[^]]+:\])+\]',
+    _sub_leading_group) # char group
+_sub_shell_basename.add(ur'\[\^?\]?(?:[^][]|\[:[^]]+:\])+\]',
+    _sub_group) # char group
+_sub_shell_basename.add(ur'\\.', ur'\&') # keep anything backslashed
+_sub_shell_basename.add(ur'[(){}|^$+.]', ur'\\&') # escape specials
+_sub_shell_basename.add(ur'^\*\.', ur'[^.].*\.') # *. after ^
+_sub_shell_basename.add(ur'^\*', ur'(?:[^.].*)?') # * after ^
+_sub_shell_basename.add(ur'\*', ur'.*') # * elsewhere
+_sub_shell_basename.add(ur'^\?', ur'[^.]') # ? after ^
+_sub_shell_basename.add(ur'\?', ur'.') # ? elsewhere
+
+
+def _sub_shell_extension(pattern):
+    return _sub_shell_basename(pattern[2:])
+
+
+class Globster(object):
+    """A simple wrapper for a set of glob patterns.
+
+    Provides the capability to search the patterns to find a match for
+    a given filename (including the full path).
+
+    Patterns are translated to regular expressions to expidite matching.
+
+    The regular expressions for multiple patterns are aggregated into 
+    a super-regex containing groups of up to 99 patterns.  
+    The 99 limitation is due to the grouping limit of the Python re module.
+    The resulting super-regex and associated patterns are stored as a list of
+    (regex,[patterns]) in _regex_patterns.
+    
+    For performance reasons the patterns are categorised as extension patterns
+    (those that match against a file extension), basename patterns
+    (those that match against the basename of the filename),
+    and fullpath patterns (those that match against the full path).
+    The regexs used for extensions and basenames are relatively simpler 
+    and therefore faster to perform than the fullpath patterns.
+
+    Also, the extension patterns are more likely to find a match and 
+    so are matched first, then the basename patterns, then the fullpath
+    patterns.
+    """
+    def __init__(self, patterns):
+        self._regex_patterns = []
+        path_patterns = []
+        base_patterns = []
+        ext_patterns = []
+        for pat in patterns:
+            if pat.startswith(u'RE:') or u'/' in pat:
+                path_patterns.append(pat)
+            elif pat.startswith(u'*.'):
+                ext_patterns.append(pat)
+            else:
+                base_patterns.append(pat)
+        self._add_patterns(ext_patterns,_sub_shell_extension,
+            prefix=r'(?:.*/)?(?!.*/)(?:[^.].*\.)')
+        self._add_patterns(base_patterns,_sub_shell_basename, 
+            prefix=r'(?:.*/)?(?!.*/)')
+        self._add_patterns(path_patterns,_sub_shell) 
+
+    def _add_patterns(self, patterns, translator, prefix=''):
+        while patterns:
+            grouped_rules = ['(%s)' % translator(pat) for pat in patterns[:99]]
+            joined_rule = '%s(?:%s)$' % (prefix, '|'.join(grouped_rules))
+            self._regex_patterns.append((re.compile(joined_rule, re.UNICODE), 
+                patterns[:99]))
+            patterns = patterns[99:]
+
+    def match(self, filename):
+        """Searches for a pattern that matches the given filename.
+        
+        :return A matching pattern or None if there is no matching pattern.
+        """
+        for regex, patterns in self._regex_patterns:
+            match = regex.match(filename)
+            if match:
+                return patterns[match.lastindex -1]
+        return None
+        

=== modified file 'bzrlib/ignores.py'
--- bzrlib/ignores.py	2006-11-02 18:27:24 +0000
+++ bzrlib/ignores.py	2006-11-13 14:13:56 +0000
@@ -88,7 +88,7 @@
     '*.o',
     '*.py[co]',
     '*.so',
-    '*.sw[nop]',
+    '.*.sw[nop]',
     '*~',
     '.#*',
     '[#]*#',

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-11-10 22:49:34 +0000
+++ bzrlib/tests/__init__.py	2006-11-13 14:13:56 +0000
@@ -1690,6 +1690,7 @@
                    'bzrlib.tests.test_escaped_store',
                    'bzrlib.tests.test_fetch',
                    'bzrlib.tests.test_ftp_transport',
+                   'bzrlib.tests.test_glob',
                    'bzrlib.tests.test_generate_ids',
                    'bzrlib.tests.test_gpg',
                    'bzrlib.tests.test_graph',

=== added file 'bzrlib/tests/test_glob.py'
--- bzrlib/tests/test_glob.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_glob.py	2006-11-13 14:13:56 +0000
@@ -0,0 +1,204 @@
+# Copyright (C) 2006 Canonical Ltd
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+from bzrlib.glob import (
+        Globster
+        )
+
+
+class TestGlobster(TestCase):
+
+    def assertMatch(self, matchset, glob_prefix=None):
+        for glob, positive, negative in matchset:
+            if glob_prefix:
+                glob = glob_prefix + glob
+            globster = Globster([glob])
+            for name in positive:
+                self.failUnless(globster.match(name), repr(
+                    u'name "%s" does not match glob "%s" (re=%s)' %
+                    (name, glob, globster._regex_patterns[0][0].pattern)))
+            for name in negative:
+                self.failIf(globster.match(name), repr(
+                    u'name "%s" does match glob "%s" (re=%s)' %
+                    (name, glob, globster._regex_patterns[0][0].pattern)))
+
+    def test_char_groups(self):
+        # The definition of digit this uses includes arabic digits from
+        # non-latin scripts (arabic, indic, etc.) and subscript/superscript
+        # digits, but neither roman numerals nor vulgar fractions.
+        matchset = [
+            (u'[[:digit:]]',
+             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21', u'\xb9'],
+             [u'T', u'q', u' ', u'\u8336', u'.']),
+            (u'[[:space:]]',
+             [u' ', u'\t', u'\n', u'\xa0', u'\u2000', u'\u2002'],
+             [u'a', u'-', u'\u8336', u'.']),
+            (u'[^[:space:]]',
+             [u'a', u'-', u'\u8336'],
+             [u' ', u'\t', u'\n', u'\xa0', u'\u2000', u'\u2002', u'.']),
+            (u'[[:alnum:]]',
+             [u'a', u'Z', u'\u017e', u'\u8336'],
+             [u':', u'-', u'\u25cf', u'.']),
+            (u'[^[:alnum:]]',
+             [u':', u'-', u'\u25cf'],
+             [u'a', u'.']),
+            (u'[[:ascii:]]',
+             [u'a', u'Q', u'^'],
+             [u'\xcc', u'\u8336', u'.']),
+            (u'[^[:ascii:]]',
+             [u'\xcc', u'\u8336'],
+             [u'a', u'Q', u'^', u'.']),
+            (u'[[:blank:]]',
+             [u'\t'],
+             [u'x', u'y', u'z', u'.']),
+            (u'[^[:blank:]]',
+             [u'x', u'y', u'z'],
+             [u'\t', u'.']),
+            (u'[[:cntrl:]]',
+             [u'\b', u'\t', '\x7f'],
+             [u'a', u'Q', u'\u8336', u'.']),
+            (u'[a-z]',
+             [u'a', u'q', u'f'],
+             [u'A', u'Q', u'F']),
+            (ur'[^a-z]',
+             [u'A', u'Q', u'F'],
+             [u'a', u'q', u'f']),
+            (u'[!a-z]foo',
+             [u'Afoo'],
+             [u'.foo']),
+            (ur'foo[!a-z]bar',
+             [u'fooAbar', u'foo.bar'],
+             [u'foojbar']),
+            (ur'[\x20-\x30\u8336]',
+             [u'\040', u'\044', u'\u8336'],
+             []),
+            (ur'[^\x20-\x30\u8336]',
+             [],
+             [u'\040', u'\044', u'\u8336']),
+            ]
+        self.assertMatch(matchset)
+        self.assertMatch(matchset,glob_prefix='./')
+
+    def test_regex(self):
+        self.assertMatch([
+            (ur'RE:(a|b|c+)',
+             [u'a', u'b', u'ccc'],
+             [u'd', u'aa', u'c+', u'-a']),
+            (ur'RE:(?:a|b|c+)',
+             [u'a', u'b', u'ccc'],
+             [u'd', u'aa', u'c+', u'-a']),
+            (ur'RE:(?P<a>.)(?P=a)',
+             [u'a'],
+             [u'ab', u'aa', u'aaa']),
+            ])
+
+    def test_question_mark(self):
+        self.assertMatch([
+            (u'?foo',
+             [u'xfoo', u'bar/xfoo', u'bar/\u8336foo'],
+             [u'.foo', u'bar/.foo', u'bar/foo', u'foo']),
+            (u'foo?bar',
+             [u'fooxbar', u'foo.bar', u'foo\u8336bar', u'qyzzy/foo.bar'],
+             [u'foo/bar']),
+            (u'foo/?bar',
+             [u'foo/xbar', u'foo/\u8336bar'],
+             [u'foo/.bar', u'foo/bar', u'bar/foo/xbar']),
+            ])
+
+    def test_asterisk(self):
+        self.assertMatch([
+            (u'x*x',
+             [u'xx', u'x.x', u'x\u8336..x', u'\u8336/x.x', u'x.y.x'],
+             [u'x/x', u'bar/x/bar/x', u'bax/abaxab']),
+            (u'foo/*x',
+             [u'foo/x', u'foo/bax', u'foo/a.x'],
+             [u'foo/.x', u'foo/.q.x', u'foo/bar/bax']),
+            (u'*/*x',
+             [u'\u8336/x', u'foo/x', u'foo/bax', u'x/a.x'],
+             [u'.foo/x', u'\u8336/.x', u'foo/.q.x', u'foo/bar/bax']),
+            (u'f*',
+             [u'foo', u'foo.bar'],
+             [u'.foo', u'foo/bar', u'foo/.bar']),
+            (u'*bar',
+             [u'bar', u'foobar', ur'foo\nbar', u'foo.bar', u'foo/bar', 
+              u'foo/foobar', u'foo/f.bar'],
+             [u'.bar', u'foo/.bar']),
+            ])
+
+    def test_leading_dotslash(self):
+        self.assertMatch([
+            (u'./foo',
+             [u'foo'],
+             [u'\u8336/foo', u'barfoo', u'x/y/foo']),
+            (u'./f*',
+             [u'foo'],
+             [u'foo/bar', u'foo/.bar', u'x/foo/y']),
+            ])
+
+    def test_leading_stardot(self):
+        self.assertMatch([
+            (u'*.x',
+             [u'foo/bar/baz.x', u'\u8336/Q.x', u'foo.y.x'],
+             [ u'.foo.x', u'bar/.foo.x', u'.x']),
+            (u'foo/*.bar',
+             [u'foo/b.bar', u'foo/a.b.bar'],
+             [u'foo/.bar', u'foo/bar']),
+            (u'*.~*',
+             [u'foo.py.~1~'],
+             [u'.foo.py.~1~']),
+            ])
+
+    def test_end_anchor(self):
+        self.assertMatch([
+            (u'*.333',
+             [u'foo.333'],
+             [u'foo.3']),
+            (u'*.3',
+             [u'foo.3'],
+             [u'foo.333']),
+            ])
+
+    def test_mixed_globs(self):
+        """tests handling of combinations of path type matches.
+
+        The types being extension, basename and full path.
+        """
+        patterns = [ u'*.foo', u'.*.swp', u'./*.png']
+        globster = Globster(patterns)
+        self.assertEqual(u'*.foo', globster.match('bar.foo'))
+        self.assertEqual(u'./*.png', globster.match('foo.png'))
+        self.assertEqual(None, globster.match('foo/bar.png'))
+        self.assertEqual(u'.*.swp', globster.match('foo/.bar.py.swp'))
+
+    def test_large_globset(self):
+        """tests that the globster can handle a large set of patterns.
+
+        Large is defined as more than supported by python regex groups, 
+        i.e. 99.
+        This test assumes the globs are broken into regexs containing 99
+        groups.
+        """
+        patterns = [ u'*.%03d' % i for i in xrange(0,300) ]
+        globster = Globster(patterns)
+        # test the fence posts
+        for x in (0,98,99,197,198,296,297,299):
+            filename = u'foo.%03d' % x
+            self.assertEqual(patterns[x],globster.match(filename))
+        self.assertEqual(None,globster.match('foobar.300'))
+

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-11-02 22:48:49 +0000
+++ bzrlib/tests/test_workingtree.py	2006-11-13 14:13:56 +0000
@@ -268,76 +268,6 @@
         file_id = self.applyDeprecated(zero_thirteen, workingtree.gen_root_id)
         self.assertStartsWith(file_id, 'tree_root-')
         
-    def test__translate_ignore_rule(self):
-        tree = self.make_branch_and_tree('.')
-        # translation should return the regex, the number of groups in it,
-        # and the original rule in a tuple.
-        # there are three sorts of ignore rules:
-        # root only - regex is the rule itself without the leading ./
-        self.assertEqual(
-            "(rootdirrule$)", 
-            tree._translate_ignore_rule("./rootdirrule"))
-        # full path - regex is the rule itself
-        self.assertEqual(
-            "(path\\/to\\/file$)",
-            tree._translate_ignore_rule("path/to/file"))
-        # basename only rule - regex is a rule that ignores everything up
-        # to the last / in the filename
-        self.assertEqual(
-            "((?:.*/)?(?!.*/)basenamerule$)",
-            tree._translate_ignore_rule("basenamerule"))
-
-    def test__combine_ignore_rules(self):
-        tree = self.make_branch_and_tree('.')
-        # the combined ignore regexs need the outer group indices
-        # placed in a dictionary with the rules that were combined.
-        # an empty set of rules
-        # this is returned as a list of combined regex,rule sets, because
-        # python has a limit of 100 combined regexes.
-        compiled_rules = tree._combine_ignore_rules([])
-        self.assertEqual([], compiled_rules)
-        # one of each type of rule.
-        compiled_rules = tree._combine_ignore_rules(
-            ["rule1", "rule/two", "./three"])[0]
-        # what type *is* the compiled regex to do an isinstance of ?
-        self.assertEqual(3, compiled_rules[0].groups)
-        self.assertEqual(
-            {0:"rule1",1:"rule/two",2:"./three"},
-            compiled_rules[1])
-
-    def test__combine_ignore_rules_grouping(self):
-        tree = self.make_branch_and_tree('.')
-        # when there are too many rules, the output is split into groups of 100
-        rules = []
-        for index in range(198):
-            rules.append('foo')
-        self.assertEqual(2, len(tree._combine_ignore_rules(rules)))
-
-    def test__get_ignore_rules_as_regex(self):
-        tree = self.make_branch_and_tree('.')
-        # Setup the default ignore list to be empty
-        ignores._set_user_ignores([])
-
-        # some plugins (shelf) modifies the DEFAULT_IGNORE list in memory
-        # which causes this test to fail so force the DEFAULT_IGNORE
-        # list to be empty
-        orig_default = bzrlib.DEFAULT_IGNORE
-        # Also make sure the runtime ignore list is empty
-        orig_runtime = ignores._runtime_ignores
-        try:
-            bzrlib.DEFAULT_IGNORE = []
-            ignores._runtime_ignores = set()
-
-            self.build_tree_contents([('.bzrignore', 'CVS\n.hg\n')])
-            reference_output = tree._combine_ignore_rules(
-                                    set(['CVS', '.hg']))[0]
-            regex_rules = tree._get_ignore_rules_as_regex()[0]
-            self.assertEqual(len(reference_output[1]), regex_rules[0].groups)
-            self.assertEqual(reference_output[1], regex_rules[1])
-        finally:
-            bzrlib.DEFAULT_IGNORE = orig_default
-            ignores._runtime_ignores = orig_runtime
-
 
 class InstrumentedTree(object):
     """A instrumented tree to check the needs_tree_write_lock decorator."""

=== modified file 'bzrlib/tests/workingtree_implementations/test_is_ignored.py'
--- bzrlib/tests/workingtree_implementations/test_is_ignored.py	2006-10-05 05:37:25 +0000
+++ bzrlib/tests/workingtree_implementations/test_is_ignored.py	2006-11-13 14:13:56 +0000
@@ -30,6 +30,11 @@
         self.build_tree_contents([
             ('.bzrignore', './rootdir\n'
                            'randomfile*\n'
+                           '*bar\n'
+                           '?foo\n'
+                           '*.~*\n'
+                           'dir1/*f1\n'
+                           'dir1/?f2\n'
                            'path/from/ro?t\n'
                            'unicode\xc2\xb5\n' # u'\xb5'.encode('utf8')
                            'dos\r\n'
@@ -63,6 +68,23 @@
         self.assertEqual('dos', tree.is_ignored('dos'))
         self.assertEqual(None, tree.is_ignored('dosfoo'))
 
+        self.assertEqual('*bar', tree.is_ignored('foobar'))
+        self.assertEqual('*bar', tree.is_ignored(r'foo\nbar'))
+        self.assertEqual('*bar', tree.is_ignored('bar'))
+        self.assertEqual(None, tree.is_ignored('.bar'))
+
+        self.assertEqual('?foo', tree.is_ignored('afoo'))
+        self.assertEqual(None, tree.is_ignored('.foo'))
+
+        self.assertEqual('*.~*', tree.is_ignored('blah.py.~1~'))
+
+        self.assertEqual('dir1/*f1', tree.is_ignored('dir1/foof1'))
+        self.assertEqual('dir1/*f1', tree.is_ignored('dir1/f1'))
+        self.assertEqual(None, tree.is_ignored('dir1/.f1'))
+
+        self.assertEqual('dir1/?f2', tree.is_ignored('dir1/ff2'))
+        self.assertEqual(None, tree.is_ignored('dir1/.f2'))
+
         # Blank lines and comments should be ignored
         self.assertEqual(None, tree.is_ignored(''))
         self.assertEqual(None, tree.is_ignored('test/'))
@@ -137,12 +159,13 @@
     def test_mixed_is_ignored(self):
         tree = self.make_branch_and_tree('.')
         ignores._set_user_ignores(['*.py[co]', './.shelf'])
-        self.build_tree_contents([('.bzrignore', './rootdir\n*.swp\n')])
+        self.build_tree_contents([('.bzrignore', './rootdir\n.*.swp\n')])
 
         self.assertEqual('*.py[co]', tree.is_ignored('foo.pyc'))
         self.assertEqual('./.shelf', tree.is_ignored('.shelf'))
         self.assertEqual('./rootdir', tree.is_ignored('rootdir'))
-        self.assertEqual('*.swp', tree.is_ignored('.foo.py.swp'))
+        self.assertEqual(None, tree.is_ignored('foo.py.swp'))
+        self.assertEqual('.*.swp', tree.is_ignored('.foo.py.swp'))
         self.assertEqual(None, tree.is_ignored('.foo.py.swo'))
 
     def test_DEFAULT_IGNORE(self):
@@ -158,7 +181,7 @@
         self.assertEqual(None, tree.is_ignored('foo.pyc'))
 
         # Must reset the list so that it reads a new one
-        tree._ignoreset = None
+        tree.flush_ignore_list_cache()
 
         # use list.append() to get around the deprecation warnings
         list.append(bzrlib.DEFAULT_IGNORE, '*.py[co]')
@@ -177,7 +200,7 @@
             ignores._runtime_ignores = set()
             self.assertEqual(None, tree.is_ignored('foobar.py'))
 
-            tree._ignoreset = None
+            tree.flush_ignore_list_cache()
             ignores.add_runtime_ignores(['./foobar.py'])
             self.assertEqual(set(['./foobar.py']), ignores.get_runtime_ignores())
             self.assertEqual('./foobar.py', tree.is_ignored('foobar.py'))

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-11-10 22:49:34 +0000
+++ bzrlib/workingtree.py	2006-11-13 14:13:56 +0000
@@ -38,14 +38,12 @@
 
 from cStringIO import StringIO
 import os
-import re
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import collections
 from copy import deepcopy
 import errno
-import fnmatch
 import stat
 from time import time
 import warnings
@@ -55,6 +53,7 @@
     bzrdir,
     conflicts as _mod_conflicts,
     errors,
+    glob,
     generate_ids,
     ignores,
     merge,
@@ -1223,60 +1222,6 @@
                 subp = pathjoin(path, subf)
                 yield subp
 
-    def _translate_ignore_rule(self, rule):
-        """Translate a single ignore rule to a regex.
-
-        There are two types of ignore rules.  Those that do not contain a / are
-        matched against the tail of the filename (that is, they do not care
-        what directory the file is in.)  Rules which do contain a slash must
-        match the entire path.  As a special case, './' at the start of the
-        string counts as a slash in the string but is removed before matching
-        (e.g. ./foo.c, ./src/foo.c)
-
-        :return: The translated regex.
-        """
-        if rule[:2] in ('./', '.\\'):
-            # rootdir rule
-            result = fnmatch.translate(rule[2:])
-        elif '/' in rule or '\\' in rule:
-            # path prefix 
-            result = fnmatch.translate(rule)
-        else:
-            # default rule style.
-            result = "(?:.*/)?(?!.*/)" + fnmatch.translate(rule)
-        assert result[-1] == '$', "fnmatch.translate did not add the expected $"
-        return "(" + result + ")"
-
-    def _combine_ignore_rules(self, rules):
-        """Combine a list of ignore rules into a single regex object.
-
-        Each individual rule is combined with | to form a big regex, which then
-        has $ added to it to form something like ()|()|()$. The group index for
-        each subregex's outermost group is placed in a dictionary mapping back 
-        to the rule. This allows quick identification of the matching rule that
-        triggered a match.
-        :return: a list of the compiled regex and the matching-group index 
-        dictionaries. We return a list because python complains if you try to 
-        combine more than 100 regexes.
-        """
-        result = []
-        groups = {}
-        next_group = 0
-        translated_rules = []
-        for rule in rules:
-            translated_rule = self._translate_ignore_rule(rule)
-            compiled_rule = re.compile(translated_rule)
-            groups[next_group] = rule
-            next_group += compiled_rule.groups
-            translated_rules.append(translated_rule)
-            if next_group == 99:
-                result.append((re.compile("|".join(translated_rules)), groups))
-                groups = {}
-                next_group = 0
-                translated_rules = []
-        if len(translated_rules):
-            result.append((re.compile("|".join(translated_rules)), groups))
-        return result
 
     def ignored_files(self):
         """Yield list of PATH, IGNORE_PATTERN"""
@@ -1296,29 +1241,20 @@
 
         ignore_globs = set(bzrlib.DEFAULT_IGNORE)
         ignore_globs.update(ignores.get_runtime_ignores())
-
         ignore_globs.update(ignores.get_user_ignores())
-
         if self.has_filename(bzrlib.IGNORE_FILENAME):
             f = self.get_file_byname(bzrlib.IGNORE_FILENAME)
             try:
                 ignore_globs.update(ignores.parse_ignore_file(f))
             finally:
                 f.close()
-
         self._ignoreset = ignore_globs
-        self._ignore_regex = self._combine_ignore_rules(ignore_globs)
         return ignore_globs
 
-    def _get_ignore_rules_as_regex(self):
-        """Return a regex of the ignore rules and a mapping dict.
-
-        :return: (ignore rules compiled regex, dictionary mapping rule group 
-        indices to original rule.)
-        """
-        if getattr(self, '_ignoreset', None) is None:
-            self.get_ignore_list()
-        return self._ignore_regex
+    def flush_ignore_list_cache(self):
+        """Resets the cached ignore list to force a cache rebuild."""
+        self._ignoreset = None
+        self._ignoreglobster = None
 
     def is_ignored(self, filename):
         r"""Check whether the filename matches an ignore pattern.
@@ -1329,27 +1265,9 @@
         If the file is ignored, returns the pattern which caused it to
         be ignored, otherwise None.  So this can simply be used as a
         boolean if desired."""
-
-        # TODO: Use '**' to match directories, and other extended
-        # globbing stuff from cvs/rsync.
-
-        # XXX: fnmatch is actually not quite what we want: it's only
-        # approximately the same as real Unix fnmatch, and doesn't
-        # treat dotfiles correctly and allows * to match /.
-        # Eventually it should be replaced with something more
-        # accurate.
-    
-        rules = self._get_ignore_rules_as_regex()
-        for regex, mapping in rules:
-            match = regex.match(filename)
-            if match is not None:
-                # one or more of the groups in mapping will have a non-None
-                # group match.
-                groups = match.groups()
-                rules = [mapping[group] for group in 
-                    mapping if groups[group] is not None]
-                return rules[0]
-        return None
+        if getattr(self, '_ignoreglobster', None) is None:
+            self._ignoreglobster = glob.Globster(self.get_ignore_list())
+        return self._ignoreglobster.match(filename)
 
     def kind(self, file_id):
         return file_kind(self.id2abspath(file_id))

